@@ -143,11 +143,7 @@ avatar_chooser_set_connection (EmpathyAvatarChooser *self,
   tp_clear_object (&self->priv->connection);
 
   if (connection != NULL)
-    {
-      GQuark features[] = { TP_CONNECTION_FEATURE_AVATAR_REQUIREMENTS, 0 };
-      self->priv->connection = g_object_ref (connection);
-      tp_proxy_prepare_async (self->priv->connection, features, NULL, NULL);
-    }
+    self->priv->connection = g_object_ref (connection);
 }
 
 static void
@@ -819,7 +815,7 @@ avatar_chooser_set_avatar_from_pixbuf (EmpathyAvatarChooser *self,
   if (!gdk_pixbuf_save_to_buffer (pb, &buf, &size, "png", &error, NULL))
     {
       avatar_chooser_error_show (self,
-        _("Couldn't save pixbuf to png"),
+        _("Couldn't save picture to file"),
         error ? error->message : NULL);
       g_clear_error (&error);
       return;
